@@ -38,6 +38,7 @@ if T.TYPE_CHECKING:
         filebase: T.Optional[str]
         description: T.Optional[str]
         url: str
+        license: str
         subdirs: T.List[str]
         conflicts: T.List[str]
         dataonly: bool
@@ -155,6 +156,14 @@ class DependenciesHelper:
                 pass
             elif isinstance(obj, dependencies.ExternalDependency) and obj.name == 'threads':
                 pass
+            elif isinstance(obj, dependencies.InternalDependency) and all(lib.get_id() in self.metadata for lib in obj.libraries):
+                # Ensure BothLibraries are resolved:
+                if self.pub_libs and isinstance(self.pub_libs[0], build.StaticLibrary):
+                    obj = obj.get_as_static(recursive=True)
+                else:
+                    obj = obj.get_as_shared(recursive=True)
+                for lib in obj.libraries:
+                    processed_reqs.append(self.metadata[lib.get_id()].filebase)
             else:
                 raise mesonlib.MesonException('requires argument not a string, '
                                               'library with pkgconfig-generated file '
@@ -441,6 +450,7 @@ class PkgConfigModule(NewExtensionModule):
     def _generate_pkgconfig_file(self, state: ModuleState, deps: DependenciesHelper,
                                  subdirs: T.List[str], name: str,
                                  description: str, url: str, version: str,
+                                 license: str,
                                  pcfile: str, conflicts: T.List[str],
                                  variables: T.List[T.Tuple[str, str]],
                                  unescaped_variables: T.List[T.Tuple[str, str]],
@@ -519,18 +529,20 @@ class PkgConfigModule(NewExtensionModule):
                 ofile.write(f'{k}={v}\n')
             ofile.write('\n')
             ofile.write(f'Name: {name}\n')
-            if len(description) > 0:
+            if description:
                 ofile.write(f'Description: {description}\n')
-            if len(url) > 0:
+            if url:
                 ofile.write(f'URL: {url}\n')
+            if license:
+                ofile.write(f'License: {license}\n')
             ofile.write(f'Version: {version}\n')
             reqs_str = deps.format_reqs(deps.pub_reqs)
-            if len(reqs_str) > 0:
+            if reqs_str:
                 ofile.write(f'Requires: {reqs_str}\n')
             reqs_str = deps.format_reqs(deps.priv_reqs)
-            if len(reqs_str) > 0:
+            if reqs_str:
                 ofile.write(f'Requires.private: {reqs_str}\n')
-            if len(conflicts) > 0:
+            if conflicts:
                 ofile.write('Conflicts: {}\n'.format(' '.join(conflicts)))
 
             def generate_libs_flags(libs: T.List[LIBS]) -> T.Iterable[str]:
@@ -571,9 +583,9 @@ class PkgConfigModule(NewExtensionModule):
                         if isinstance(l, (build.CustomTarget, build.CustomTargetIndex)) or 'cs' not in l.compilers:
                             yield f'-l{lname}'
 
-            if len(deps.pub_libs) > 0:
+            if deps.pub_libs:
                 ofile.write('Libs: {}\n'.format(' '.join(generate_libs_flags(deps.pub_libs))))
-            if len(deps.priv_libs) > 0:
+            if deps.priv_libs:
                 ofile.write('Libs.private: {}\n'.format(' '.join(generate_libs_flags(deps.priv_libs))))
 
             cflags: T.List[str] = []
@@ -605,6 +617,7 @@ class PkgConfigModule(NewExtensionModule):
         KwargInfo('name', (str, NoneType), validator=lambda x: 'must not be an empty string' if x == '' else None),
         KwargInfo('subdirs', ContainerTypeInfo(list, str), default=[], listify=True),
         KwargInfo('url', str, default=''),
+        KwargInfo('license', str, default='', since='1.9.0'),
         KwargInfo('version', (str, NoneType)),
         VARIABLES_KW.evolve(name="unescaped_uninstalled_variables", since='0.59.0'),
         VARIABLES_KW.evolve(name="unescaped_variables", since='0.59.0'),
@@ -659,6 +672,7 @@ class PkgConfigModule(NewExtensionModule):
         filebase = kwargs['filebase'] if kwargs['filebase'] is not None else name
         description = kwargs['description'] if kwargs['description'] is not None else default_description
         url = kwargs['url']
+        license = kwargs['license']
         conflicts = kwargs['conflicts']
 
         # Prepend the main library to public libraries list. This is required
@@ -713,7 +727,7 @@ class PkgConfigModule(NewExtensionModule):
                 pkgroot_name = os.path.join('{libdir}', 'pkgconfig')
         relocatable = state.get_option('pkgconfig.relocatable')
         self._generate_pkgconfig_file(state, deps, subdirs, name, description, url,
-                                      version, pcfile, conflicts, variables,
+                                      version, license, pcfile, conflicts, variables,
                                       unescaped_variables, False, dataonly,
                                       pkgroot=pkgroot if relocatable else None)
         res = build.Data([mesonlib.File(True, state.environment.get_scratch_dir(), pcfile)], pkgroot, pkgroot_name, None, state.subproject, install_tag='devel')
@@ -722,7 +736,7 @@ class PkgConfigModule(NewExtensionModule):
 
         pcfile = filebase + '-uninstalled.pc'
         self._generate_pkgconfig_file(state, deps, subdirs, name, description, url,
-                                      version, pcfile, conflicts, variables,
+                                      version, license, pcfile, conflicts, variables,
                                       unescaped_variables, uninstalled=True, dataonly=dataonly)
         # Associate the main library with this generated pc file. If the library
         # is used in any subsequent call to the generated, it will generate a
